@@ -14,6 +14,7 @@ protocol converters, together with helper functions and classes.
 
 __all__ = ['MCP2221', 'find_devices']
 
+import os
 import hid
 import warnings
 
@@ -220,6 +221,11 @@ class MCP2221():
         """
         if self._opened:
             cmd = self._build_command(*args)
+            if os.name == 'nt':
+                # Windows HID requires additional prefix byte ReportID=0x00
+                # https://stackoverflow.com/questions/22240591/confused-by-the-report-id-when-using-hidapi-through-usb
+                # https://www.microchip.com/forums/m887066.aspx
+                cmd.insert(0, 0x00)
             self.dev.write(cmd)
             self.dev.set_nonblocking(False)
             data = self._read_response(args[0])
@@ -343,12 +349,17 @@ class MCP2221():
         gp_set = self._write(0x51)[2:10]
         cmd = bytearray(64)
         cmd[0] = 0x60
-        cmd[8:12] = [(gp_set[2*n] << 4) + (gp_set[2*n+1] << 3) for n in range(4)]
+        # set GPIO directions/values if these are relevant
+        for n in range(4):
+            if gp_set[2*n] <= 1:
+                cmd[8+n] = (gp_set[2*n] << 4) + (gp_set[2*n+1] << 3)
+        
         if code == SramDataSubcode.ChipSettings:
             idx = 2 + byte
         elif code == SramDataSubcode.GPSettings:
             idx = 8 + byte
             cmd[7] = 0x80
+        
         cmd[idx] = value
         self._write(*cmd)
 
@@ -1089,7 +1100,7 @@ class MCP2221():
         if length > 0xffff:
             raise InvalidParameterException("Data string too long.")
 
-    def i2c_write_data(self, address:bytes, data:str, i2c_mode:I2CMode = I2CMode.Start) -> None:
+    def i2c_write_data(self, address:bytes, data:bytearray, i2c_mode:I2CMode = I2CMode.Start) -> None:
         """Writes data to given I2C address.
 
         Parameters:
@@ -1102,7 +1113,7 @@ class MCP2221():
         rem_bytes = dlen
         while rem_bytes>0:
             chunk_len = min(rem_bytes, 60)
-            chunk = data[dlen-rem_bytes:dlen-rem_bytes+chunk_len].encode("utf-8")
+            chunk = data[dlen-rem_bytes:dlen-rem_bytes+chunk_len]
             while True:
                 ret = self._write(i2c_mode, dlen & 0xff, dlen>>8, address<<1, *chunk)
                 if ret[1] == 0: break
@@ -1245,7 +1256,9 @@ class MCP2221():
             # assigned with 0x60; to avoid overwriting pin value/dir, we must get
             # data with 0x51
             init = self._write(0x51)[(2+2*gpio_num):(4+2*gpio_num)]
-            value += (init[0]<<4) + (init[1]<<3)
+            if init[0] <= 1:
+                value += (init[0]<<4) + (init[1]<<3)
+            
             self._write_sram(SramDataSubcode.GPSettings, gpio_num, value)
         elif mem == MemoryType.Flash:
             self._write_flash_byte(FlashDataSubcode.GPSettings, gpio_num, [0, 1, 2], self.__byte_to_bits(value, 3))
